package com.ruoyi.system.utils;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.ruoyi.common.core.domain.entity.SysRole;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.system.mapper.SysRoleMapper;
import com.ruoyi.system.service.ISysRoleService;
import com.ruoyi.system.service.ISysUserService;

import java.util.*;
import java.util.function.BiConsumer;
import java.util.function.Function;
import java.util.stream.Collectors;

public class UserUtil {

    private static final ISysUserService sysUserService = SpringUtil.getBean(ISysUserService.class);
    private static final SysRoleMapper roleMapper = SpringUtil.getBean(SysRoleMapper.class);
    private static final ISysRoleService roleService = SpringUtil.getBean(ISysRoleService.class);

    public static List<SysUser> getListByIds(List<Long> idList) {
        if (idList.size() == 0) {
            return new ArrayList<>();
        }
        return sysUserService.getListByUserId(idList);
    }

    public static Map<Long, SysUser> getMapByIds(List<Long> idList) {
        return getListByIds(idList).stream().collect(Collectors.toMap(SysUser::getUserId, Function.identity()));
    }

    public static Map<Long, String> getNickNameMapByIds(List<Long> idList) {
        if (idList.size() == 0) {
            new HashMap<>();
        }
        return getListByIds(idList).stream().collect(Collectors.toMap(SysUser::getUserId, SysUser::getNickName));
    }

    public static String getNickNameById(Long userId) {
        return getNickNameMapByIds(Collections.singletonList(userId)).get(userId);
    }

    public static <T> void assignmentNickName(List<T> list, Function<T, Long> idMapper, BiConsumer<T, String> setNickNameMap) {
        if (list == null || list.size() == 0) {
            return;
        }
        List<Long> idList = list.stream().map(idMapper).collect(Collectors.toList());
        Map<Long, String> nickNameMapByIds = getNickNameMapByIds(idList);
        list.forEach(item -> setNickNameMap.accept(item, nickNameMapByIds.get(idMapper.apply(item))));
    }

    /**
     * 获取有权限用户列表
     */
    public static List<SysUser> getAuthUserList() {
        return sysUserService.getAuthUserList();
    }

    /**
     * 获取有权限用户id列表
     */
    public static List<Long> getAuthUserIdList() {
        return getAuthUserList().stream().map(SysUser::getUserId).collect(Collectors.toList());
    }

    /**
     * 获取用户
     */
    public static SysUser getUserInfo() {
        return sysUserService.getUserById();
    }

    /**
     * 根据角色KEY获取用户id列表
     */
    public static List<Long> getUserIdsByRoleKey(String roleKey) {
        return sysUserService.getUserIdsByRoleKey(roleKey);
    }

    /**
     * 根据角色KEY和公司id获取用户id列表
     */
    public static List<Long> getUserIdsByRoleKey(String roleKey, Long companyId) {
        return sysUserService.getUserIdsByRoleKey(roleKey, companyId);
    }

    public static List<Long> getUserIdsByRoleKeys(List<String> roleKeys, Long companyId) {
        roleKeys = roleKeys.stream().distinct().collect(Collectors.toList());
        List<Long> uidList = new ArrayList<>();
        for (String roleKey : roleKeys) {
            uidList.addAll(sysUserService.getUserIdsByRoleKey(roleKey, companyId));
        }
        return uidList.stream().distinct().collect(Collectors.toList());
    }

    /**
     * 获取用户角色列表
     */
    public static List<SysRole> getUserRoles(Long userId) {
        return roleMapper.selectRolePermissionByUserId(userId);
    }

    /**
     * 获取用户角色Key列表
     */
    public static List<String> getUserRoleKeys(Long userId) {
        return getUserRoles(userId).stream().map(SysRole::getRoleKey).distinct().collect(Collectors.toList());
    }

    public static List<Long> getUserRoleIds(Long userId) {
        return getUserRoles(userId).stream().map(SysRole::getRoleId).distinct().collect(Collectors.toList());
    }


    /**
     * 获取用户角色以及子角色
     */
    public static List<SysRole> getUserRoleAndChildRole(Long userId) {
        return recursionRoles(getUserRoles(userId), 0);
    }

    public static List<String> getUserRoleAndChildRoleKeys(Long userId) {
        return getUserRoleAndChildRole(userId).stream().map(SysRole::getRoleKey).distinct().collect(Collectors.toList());
    }

    /**
     * 递归角色以及子角色
     */
    private static List<SysRole> recursionRoles(List<SysRole> userRoles, int count) {
        count++;
        if (count >= 100) {
            throw new ServiceException("查询角色子级循环超过" + count + "次，可能成环，请检查或联系管理员！");
        }

        List<SysRole> temp = new ArrayList<>();

        for (SysRole userRole : userRoles) {
            List<SysRole> list = roleService.list(IWrapper.<SysRole>getWrapper().eq(SysRole::getParentId, userRole.getRoleId()));
            if (ObjectUtil.isEmpty(list)) {
                continue;
            }

            List<SysRole> roles = recursionRoles(list, count);
            temp.addAll(roles);
        }
        userRoles.addAll(temp);
        return userRoles;
    }

}
